function MeasOut = ChiSquaredDataFusionCov(Sens)
%--------------------------------------------------------------------------
% ChiSquaredDataFusionCov.m
%
% Form estimates of the measurand and associated uncertainties from sensor 
% estimates and associated uncertainties using the correlated chi-squared 
% data fusion method. 
%--------------------------------------------------------------------------
% Input
% Sens                   Structure defining estimates of measurand from 
%                        sensor responses: 
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m2 x nS
%                        VY        Covariance matrices associated with 
%                                  measurand estimates (at each time value)
%                                  Dimension: m2 x 1 cell (of nS x nS 
%                                             arrays)
%
% Output
% MeasOut                Structure defining estimates of measurand:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        y         Measurand estimates 
%                                  Dimension: m2 x 1
%                        uy        Standard uncertainties associated with
%                                  measurand estimates
%                                  Dimension: m2 x 1
%                        indices   Indices of consistent sensors
%                                  Dimension: m2 x 1
% -------------------------------------------------------------------------

% Display data fusion method.
%
  fprintf('\nData fusion method: Correlated chi-squared - LCS \n');
%  
% Initialize matrices for measurand estimate and uncertainties.
%
  Y       = Sens.Y;
  VY      = Sens.VY;
  t       = Sens.t;
  mS      = size(Y, 1); 
  y       = zeros(mS, 1);
  uy      = zeros(mS, 1);
  indices = cell(mS, 1);
  dispInd = 0; % indicator for displaying consistent subset warning
%
% Estimate measurand.
%
  for it = 1:mS
      YI = find(~isnan(Y(it,:)));
      if isempty(YI)
          NLCS = 0;
      else
          VYI = VY{it};
          [yLCS, uyLCS, PI] = LCSCorrelatedFull(Y(it,YI)', VYI(YI,YI));
          P = YI(PI);
          NLCS = numel(P);
      end
      if  NLCS > 0
          y(it)       = yLCS; 
          uy(it)      = uyLCS;
          indices(it) = {P};
      end
%    
% Check for case no consistent subset.
%
      if NLCS == 0
          if dispInd == 0
              txt = ['WARNING: No consistent subset found at t = ',num2str(t(it)), 's']; 
              disp(txt);
              disp('Warning will not show again for any subsequent times with no consistent subset');
              dispInd = 1;
          end
          if (it > 1)
%          
% Take measurand estimate from previous time point.
%
              y(it)       = y(it-1);
              uy(it)      = uy(it-1);
              indices(it) = {[]}; % empty set
          elseif ~isempty(YI)
%          
% Use the output from the first functioning sensor.
%
              y(it)       = Y(it,YI(1));
              sens1       = YI(1);
              vy          = VY{it};
              uy(it)      = sqrt(vy(sens1,sens1));
              indices(it) = {YI(1)};
          else
              y(it)       = NaN;
              uy(it)      = NaN;
              indices(it) = {[]}; % empty set
          end    
      end    
  end
% 
% Store estimates data in structure.
% 
  MeasOut.t       = t;
  MeasOut.y       = y; 
  MeasOut.uy      = uy; 
  MeasOut.indices = indices;
%  
% -------------------------------------------------------------------------
% End ChiSquaredDataFusionCov.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------