  function c = Count3(c, L)
% -------------------------------------------------------------------------
% Count3.m
%
% Provide index set of the next in the sequence of index sets representing 
% a choice of n distinct items from L items.
% -------------------------------------------------------------------------
% Input
%	c                      Index set for member of a sequence of index sets
%                        Dimension: n x 1
%	L                      Number of items from which n are chosen
%
% Output
% c                      Index set for next in the sequence
%                        Dimension: n x 1
% -------------------------------------------------------------------------
% There are L(L - 1)...(L - n + 1)/(1*2* ... *n) sets in the sequence.
%
% For example, the sequence when n = 3 and L = 5 is
%      1     2     3
%      1     2     4
%      1     2     5
%      1     3     4
%      1     3     5
%      1     4     5
%      2     3     4
%      2     3     5
%      2     4     5
%      3     4     5
% and when c as an input parameter is [1 4 5], c as an output parameter
% is [2 3 4].
%
% Before the first call of count3, c should be initialized to any set in
% the sequence or to the set immediately preceding the first member of
% a complete sequence.
%
% To produce a complete sequence, or a leading sub-sequence, c should be 
% initialized to [1, 2, ..., n - 1, n - 1], the index set preceding the
% first in the sequence. For the above sequence, c would be initialized 
% to [1, 2, 2].
%
% Count3 is based on simulating a specialized digital counter, with
% the following properties:
%   (1) the counter has n digits;
%   (2) each digit lies between 1 and L;
%   (3) no digit in the counter is repeated;
%   (4) when the counter is regarded as displaying an n-digit set of 
%       numbers in arithmetic to base L, the sequence produced is 
%       strictly increasing.
%
% No check on the input parameters is made. A check should be made
% from the calling function.
%
% One application of count3 is in fully enumerating all subsets of size
% n from a set of size L.
% -------------------------------------------------------------------------

  n = length(c);
  k = n;
  while c(k) == L - n + k
      k = k - 1;
  end
  c(k) = c(k) + 1;
  c(k + 1:n) = c(k) + 1:c(k) + n - k;
%
% -------------------------------------------------------------------------
% End Count3.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------