function MeasIn = FourierSeries(workbookpath, measurandworksheet)
% -------------------------------------------------------------------------
% FourierSeries.m
%
% Read information describing a measurand comprising a sum of sinusoidal 
% components from Excel worksheet, generate measurand and store in 
% structure. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% measurandworksheet     Worksheet containing measurand data 
%
% Output
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%                        name      Measurand name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
%                        f0        Frequency values for use by Kalman 
%                                  filter
%                                  Dimension: nF x 1
% -------------------------------------------------------------------------

% Display measurand simulation method and name of worksheet. 
%
  fprintf('\nType of measurand simulation: Fourier series \n');
  fprintf(['Worksheet: %', num2str(length(measurandworksheet)), 's \n'], ... 
      measurandworksheet); 
%  
% Read measurand information from worksheet. 
% 
  [num, qName] = xlsread(workbookpath, measurandworksheet, 'B4'); %#ok<ASGLU>
  [num, qUnit] = xlsread(workbookpath, measurandworksheet, 'B5'); %#ok<ASGLU>
  if isempty(qUnit)
      qUnit = {'unit'};
  end
  if isempty(qName)
      qName = {'Measurand'};
      yStr = ['Quantity/', qUnit{1}]; % string for plot axis
  else
      yStr = [qName{1}, '/', qUnit{1}];
  end  
%  
  SetUp    = xlsread(workbookpath, measurandworksheet, 'B9:B11');
  f1       = SetUp(1); % sampling frequency
  D        = SetUp(2); % signal duration
  nF       = SetUp(3); % number of sinusoidal components  
  rangeStr = ['C9:H', num2str(8+nF)];
  sigma    = xlsread(workbookpath, measurandworksheet, 'I9');
  data     = xlsread(workbookpath, measurandworksheet, rangeStr); 
%
% Generate measurand and store in structure.
%
  MeasIn      = GenerateFourierSeries(f1, D, nF, data, sigma);
  MeasIn.name = qName;
  MeasIn.yStr = yStr;
%  
% -------------------------------------------------------------------------
% End FourierSeries.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------