function MeasIn = GenerateFourierSeries(fs, T, nF, data, rho)
% -------------------------------------------------------------------------
% GenerateFourierSeries.m
%
% Draw coefficients describing measurand from their associated 
% distributions, generate measurand and store in structure.
% -------------------------------------------------------------------------
% Input
% f1                     Sampling frequency (Hz)
% D                      Duration of signal (s)
% nF                     Number of harmonics
% data                   Estimates of frequencies and associated standard
%                        uncertainties, estimates of amplitudes and 
%                        associated standard uncertainties, estimates of 
%                        phases and associated standard uncertainties
%                        Dimension: nF x 6
% rho                    Additive noise level for ALL sinusoidal components 
%
% Output
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%                        f0        Frequency values for use by Kalman 
%                                  filter
%                                  Dimension: nF x 1
% -------------------------------------------------------------------------

% Draw frequency, amplitude and phase values describing measurand from 
% their associated distributions.
%
  Rf   = GenerateRandomDraws('N', 1, nF); 
  F    = data(:, 1).*(1 + data(:, 2).*Rf);   % frequency values
  RA   = GenerateRandomDraws('N', 1, nF); 
  A    = data(:, 3).*(1 + data(:, 4).*RA);   % amplitude values
  Rphi = GenerateRandomDraws('N', 1, nF); 
  P    = data(:, 5).*(1 + data(:, 6).*Rphi); % phase values
%
% Generate measurand. 
%
  t = (1/fs:1/fs:(T))'; % time values 
  m1 = length(t); 
  y = zeros(m1, 1);
  for k = 1:nF
      y = y + A(k)*sin(2*pi*F(k)*t + P(k)*pi/180);
  end
  Re = GenerateRandomDraws('N', 1, m1);
  y = y + rho*Re;
%
% Store measurand data in structure.
%
  MeasIn.t  = t;
  MeasIn.y  = y;
  MeasIn.f0 = F; % frequency used in Kalman filter implementation
%  
% -------------------------------------------------------------------------
% End GenerateFourierSeries.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------