function MeasIn = GeneratePiecewiseLinear(f1, D, T, alphabeta, ualphabeta, sigma)
% -------------------------------------------------------------------------
% GeneratePiecewiseLinear.m
%
% Draw coefficients describing measurand from their associated 
% distributions, generate measurand and store in structure.
% -------------------------------------------------------------------------
% Input
% f1                     Sampling frequency (Hz)
% D                      Duration of signal (s)
% T                      Start times for segments (s)
%                        Dimension: nL x 1
% alphabeta              Estimates of 'intercept' and 'gradient' components
%                        Dimension: nL x 2
% ualphabeta             Standard uncertainties associated with estimates
%                        of 'intercept' and 'gradient' components 
%                        Dimension: nL x 2
% sigma                  Additive noise levels
%                        Dimension: nL x 1
%
% Output
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
% -------------------------------------------------------------------------

% Draw components describing measurand from their associated 
% distributions.
%
  nL      = size(alphabeta, 1);
  alpha   = alphabeta(:,1); 
  beta    = alphabeta(:,2); 
  uralpha = ualphabeta(:,1); 
  urbeta  = ualphabeta(:,2); 
  Ra      = GenerateRandomDraws('N', 1, nL);
  alphas  = alpha.*(1 + Ra.*uralpha); % 'intercept' components 
  Rb      = GenerateRandomDraws('N', 1, nL);
  betas   = beta.*(1 + Rb.*urbeta); % 'gradient' components 
  T       = [T; T(1) + D]; % end time appended to start times 
  t       = (T(1) + 1/f1:1/f1:(T(1) + D))'; % time values 
  m1      = length(t);
%
% Generate measurand. 
%
  sumN     = 0;
  C        = zeros(m1,3);
  DT       = zeros(m1,1);
  for k = 1:nL
      intN  = floor(f1*(T(k + 1) - T(k))); 
      C(sumN + 1:sumN + intN,:) = ones(intN, 1)*[alphas(k), betas(k), sigma(k)];
      DT(sumN + 1:sumN + intN)  = t(sumN + 1:sumN + intN) - T(k);
      sumN = sumN + intN;
  end
  Re = GenerateRandomDraws('N', 1, m1);
  y = C(:,1) + C(:,2).*DT + Re.*C(:,3);
%
% Store measurand data in structure.
%
  MeasIn.t = t;
  MeasIn.y = y;
%  
% -------------------------------------------------------------------------  
% End GeneratePiecewiseLinear.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------