function R = GenerateRandomDraws(type, sigma, m)
% -------------------------------------------------------------------------
% GenerateRandomDraws.m
%
% Generate draws from a Gaussian or rectangular distribution using the 
% Wichmann-Hill and Box-Muller algorithms.
% -------------------------------------------------------------------------
% Input 
% type                   Type of distribution ('N' for Gaussian, 'R' for
%                        rectangular) 
% sigma                  Standard deviation
% m                      Number of draws 
% 
% Output 
% R                      Random draws 
%                        Dimension: m x 1 
% -------------------------------------------------------------------------

  global g_wh_seed % seed for random number generation 
% 
  if type == 'R'
      type = 1;
  elseif type == 'N' 
      type = 0;
  end    
  if (sigma ~= 0.0)
      [R, g_wh_seed] = WHSampleData(type, sigma, m, g_wh_seed);
  else
      R = zeros(m,1);
  end    
end
%
% -------------------------------------------------------------------------
% End GenerateRandomDraws.m
% -------------------------------------------------------------------------
%
function [R, state] = WHSampleData(dist, sigma, m, state)
% -------------------------------------------------------------------------
% WHSampleData.m
%
% Generate draws from a Gaussian or rectangular distribution, interfacing 
% to Wichmann-Hill DLL.
% -------------------------------------------------------------------------
% Input 
% dist                   Type of distribution (0 for Gaussian, 1 for
%                        rectangular) 
% sigma                  Standard deviation
% m                      Number of draws 
% state                  Input state 
%                        Dimension: 1 x 4 
% 
% Output 
% R                      Random draws 
%                        Dimension: m x 1 
% state                  Output state 
%                        Dimension: 1 x 4 
% -------------------------------------------------------------------------
%
  libname = 'WH_MVS';
  if ~libisloaded(libname)
      loadlibrary(libname, libname);
  end
  R = zeros(m, 1);
  p_R = libpointer('doublePtr', R);
  seed_in = zeros(4, 1, 'int32');
  seed_in(1:4) = state(1:4);
  p_seed_in = libpointer('int32Ptr', seed_in);
  seed_out = zeros(4, 1, 'int32');
  p_seed_out = libpointer('int32Ptr', seed_out);
  idist = zeros(1, 1, 'int32');
  idist(1,1) = dist;
  p_idist = libpointer('int32Ptr', idist); 
  p_sigma = libpointer('doublePtr', sigma);
  im = zeros(1, 1, 'int32');
  im(1,1) = m;
  p_im = libpointer('int32Ptr', im); 
  [~, ~, ~, ~, R, seed_out] = calllib(libname, 'WH_SAMPLEDATA', ... 
      p_idist, p_sigma, p_im, p_seed_in, p_R, p_seed_out); 
  state = seed_out; 
end
%
% -------------------------------------------------------------------------
% End WHSampleData.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------