function [Obs, Phi] = GenerateSystemResponse(t, y, yData)
% -------------------------------------------------------------------------
% GenerateSystemResponse.m
%
% Generate a second order system response to a disturbance and store 
% response and state propagation matrices in structures. 
% -------------------------------------------------------------------------
% Input
% t                      Time values
%                        Dimension: m1 x 1
% y                      Measurand values 
%                        Dimension: m1 x 1
% yData                  Parameters (resonance frequency and Q-factor) 
%                        describing second order system response
%                        Dimension: 1 x 2
%
% Output
% Obs                    Structure defining observed system response:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         System response values 
%                                  Dimension: m1 x 1
% Phi                    Structure defining state propagation matrices:
%                        A         First state propagation matrix
%                                  Dimension: 2 x 2
%                        B         Second state propagation matrix
%                                  Dimension: 2 x 1
%                        C         Third state propagation matrix
%                                  Dimension: 1 x 2
%                        D         Fourth state propagation matrix
%                                  Dimension: 1 x 1
% -------------------------------------------------------------------------

% The model for the process is a second order differential equation.
%
  fr = yData(1);    % resonant frequency
  Qr = yData(2);    % Q-factor for damping
  wr = 2*pi*fr;
  dr = -wr/(2*Qr);
  c1 = -dr;         % alpha_1 = 2*c_1
  c0 = wr^2 + dr^2; % alpha_0 = c0
%
% Express the differential equation model (approximately) as a difference
% equation model using a bilinear transformation:
% s = (2/dt)(1- z^-1)/(1 + z^-1).
% This is equivalent to calling Matlab's bilinear function.
%
  dt = t(2) - t(1); % sampling interval
  b0 = dt^2/(c0*dt^2 + 4*c1*dt + 4);
  b1 = 2*dt^2/(c0*dt^2 + 4*c1*dt + 4);
  b2 = dt^2/(c0*dt^2 + 4*c1*dt + 4);
  a0 = (c0*dt^2 - 4*c1*dt + 4)/(c0*dt^2 + 4*c1*dt + 4);
  a1 = (2*c0*dt^2 - 8)/(c0*dt^2 + 4*c1*dt + 4);
%
% Express the difference equation model as a state-space model.
%
  A = [0, 1; -a0, -a1];
  B = [0; 1];
  C = [(b0-a0*b2), (b1-a1*b2)];
  D = b2; 
%
% The second order system is driven by a disturbance, and generates an 
% output. 
%
  m1 = length(t);
%
% Generate disturbance and process output based on state-space model.
%
  y1    = zeros(m1, 1);
  x     = zeros(2, 1);
  y1(1) = C*x + D*y(1);
%
  for k = 2:m1
%
% Update state-vector.
%
      x = A*x + B*y(k - 1);
%
% Update process output.
%
      y1(k) = C*x + D*y(k);
% 
  end
%
% Store system response data and state propagation matrices in structures.
%
  Obs.t = t;
  Obs.y = y1;
  Phi.A = A;
  Phi.B = B;
  Phi.C = C;
  Phi.D = D;
%  
% -------------------------------------------------------------------------  
% End GenerateSystemResponse.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------