function MeasOut = KalmanFilterDataFusion(Sens, Phi, f0)
% -------------------------------------------------------------------------
% KalmanFilterDataFusion.m
%
% Form estimate of the measurand and associated uncertainties using the 
% Kalman filter data fusion method. 
% -------------------------------------------------------------------------
% Input
% Sens                   Structure defining estimates of measurand from 
%                        sensor responses:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  measurand estimates
%                                  Dimension: m2 x nS
% Phi                    Structure defining state propagation matrices:
%                        A         First state propagation matrix
%                                  Dimension: 2 x 2
%                        B         Second state propagation matrix
%                                  Dimension: 2 x 1
%                        C         Third state propagation matrix
%                                  Dimension: 1 x 2
%                        D         Fourth state propagation matrix
%                                  Dimension: 1 x 1
% f0                     Frequency value describing measurand
%                                  Dimension: 1 x 1
%
% Output
% MeasOut                Structure defining estimate of measurand:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        y         Measurand estimates
%                                  Dimension: m2 x 1
%                        uy        Standard uncertainties associated with
%                                  estimates
%                                  Dimension: m2 x 1
%                        indices   Indices of consistent sensors
%                                  Dimension: m2 x 1
% -------------------------------------------------------------------------

% Display data fusion method.
%
  fprintf('\nData fusion method: Kalman filter \n');
%
% Check that measurand contains a single frequency (harmonic) only.
%
  if length(f0) > 1
      error('Example 2 - Measurand must contain a single frequency only')
  end
%
% Initialize variables.
%
  t   = Sens.t;
  Ys  = Sens.Y;
  uYs = Sens.uY;
  [m2, nS] = size(Ys); % Number of data points
%  
% Initialize variables for Kalman filter.
%
  sc  = [sin(2*pi*f0*t), cos(2*pi*f0*t)];
  A   = Phi.A;
  B   = Phi.B;
  C   = Phi.C;
  D   = Phi.D;
  nU  = size(sc, 2);
  n   = length(C);
%
% The state vector xp is of dimension 4 x 1. The last two components of the
% state vector are functions of the unknown amplitude and phase of the
% measurand.
%
  uP  = [100, 100, 1, 1]'; % standard uncertainties associated with state 
  G   = ones(n + nU, 1);   % G and QL define the process covariance matrix
  QL  = 0;  
  Sp0 = diag(uP);          % square root factor of state covariance matrix 
%
% Initialize Kalman filter estimates.
%
  yKF     = zeros(m2, 1);
  uyKF    = zeros(m2, 1);
  indices = cell(m2, 1);
%
% First time value (i2 = 1). 
%
  YI = find(~isnan(Ys(1,:))); 
  indices(1) = {YI};
  [muP, nuP] = size(uP); 
  RuP = GenerateRandomDraws('N', 1, muP*nuP); 
  RuP = reshape(RuP, muP, nuP); 
  xp = RuP.*uP; 
  Sp   = diag(uP);
  sc2  = sc(1,:);
  RL   = diag(uYs(1,:));                
  H    = ones(nS, 1)*[C, D*sc(1,:)]; % design matrix
  [xp, Sp] = UpdateMeasSRCF(xp, Sp, Ys(1,:)', H, RL);
  yKF(1)   = [0, 0, sc2]*xp;
  uyKF(1)  = sqrt([0, 0, sc2]*(Sp*Sp')*[0, 0, sc2]');
%
% Loop over remaining time values.
%      
  for i2 = 2:m2
      RL = diag(uYs(i2,:));
      PhiKF = [A, B*sc(i2 - 1,:); [zeros(n), eye(nU)]]; % propagation matrix
      H = ones(nS, 1)*[C, D*sc(i2,:)];                  % design matrix
      [xp, Sp, I] = RunKalmanFilter(xp, Sp, PhiKF, G, QL, Ys(i2,:)', H, RL, Sp0);
%      
% Estimates of process output and disturbance.
%
      sc2         = sc(i2,:);
      yKF(i2)     = [0, 0, sc2]*xp;
      uyKF(i2)    = sqrt([0, 0, sc2]*(Sp*Sp')*[0, 0, sc2]');
      indices{i2} = I';
  end
%
  MeasOut.t       = Sens.t;
  MeasOut.y       = yKF; 
  MeasOut.uy      = uyKF;
  MeasOut.indices = indices;
%  
% -------------------------------------------------------------------------
% End KalmanfilterDataFusion.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------