function [y, uy, Q] = LCSCorrelatedFull(x, Vx)
% -------------------------------------------------------------------------
% LCSCorrelatedFull.m
%
% Core computation for determining largest consistent subset (LCS) for 
% correlated measurements.
% -------------------------------------------------------------------------
% Input
% x                      Measured values 
%                        Dimension: N x 1
% Vx                     Covariance matrix associated with measured values
%                        Dimension: N x N
%
% Output
% y                      Consensus solution
% uy                     Standard uncertainty assoicated with solution
% Q                      Indices defining LCS
% -------------------------------------------------------------------------

% Initialization for all subsets.
%
  N  = length(x);
  y  = [];
  uy = [];
  Q  = [];
%
% For each size k of subset, search for a subset of the measurement 
% results that is consistent.
%
  for k = 2:N
%
% Initialisation for subsets of size k.
%
      yk    = [];
      uyk   = [];
      Qk    = [];
      chi2k = [];
%
% Define value for chi-squared test.
%
      chi2test = ChiSquared(k - 1);
%
% Evaluate number of subsets of size k.
%
      Nk = factorial(N)/(factorial(N - k)*factorial(k));
%
% Initialise index subset defining a subset.
%
      Qkj = [(1:k-1), k-1];
%
% For each subset ...
%
      for j = 1:Nk
%
% Generate index set defining a subset in the sequence.
%
          Qkj = Count3(Qkj, N);
%
% Evaluate generalized weighted mean and corresponding observed chi-squared 
% value.
%
          [ykj, uykj, chi2kj] = GenWhtMean(Qkj, x, Vx);
%
% Perform consistency test.
%
          if chi2kj < chi2test
              Qk    = [Qk, Qkj']; %#ok<AGROW>
              yk    = [yk, ykj]; %#ok<AGROW>
              uyk   = [uyk, uykj]; %#ok<AGROW>
              chi2k = [chi2k, chi2kj]; %#ok<AGROW>
          end
      end
%
% Find subset of size k that is consistent (if there is one), and gives the
% smallest observed chi-squared value.
%
      if ~isempty(chi2k)
          kmin = find(chi2k == min(chi2k), 1, 'first');
          y    = yk(kmin);
          uy   = uyk(kmin);
          Q    = Qk(:,kmin);
      else
          return
      end
  end
end
%
% -------------------------------------------------------------------------
% End LCSCorrelatedFull.m
% -------------------------------------------------------------------------
%
function [y, uy, chi2obs] = GenWhtMean(Q, x, Vx)
% -------------------------------------------------------------------------
% GenWhtMean.m
%
% Compute the generalized weighted mean, its associated standard 
% uncertainty and observed chi-squared value.
% -------------------------------------------------------------------------
% Input
% Q                      Index set
%                        Dimension: n x 1
% x                      Measured values 
%                        Dimension: N x 1
% Vx                     Covariance matrix associated with measured values
%                        Dimension: N x N
%
% Output
% y                      Consensus solution
% uy                     Standard uncertainty assoicated with solution
% chi2obs                Observed chi-squared value
% -------------------------------------------------------------------------
% 
% Define subset (z, Vz) of measurement results (x, Vx) defined by indices Q.
%
  n  = length(Q);
  z  = x(Q);			
  Vz = Vx(Q,Q);
%
% Evaluate generalized weighted mean y.
%
  A = ones(n,1);
  L = chol(Vz)';
  zt = L\z;
  At = L\A;
  y  = At\zt;
  uy = sqrt(inv(At'*At));
%
% Evaluate corresponding observed chi-squared value.
%
  chi2obs = sum((zt - At*y).^2);
end 
%
% -------------------------------------------------------------------------
% End GenWhtMean.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------