  function [NLCS, P, yLCS, uyLCS] = LCSFullEnumerate(x, ux, chi2thresh)
% -------------------------------------------------------------------------
% LCSFullEnumerate.m
% 
% Determine largest consistent subset (LCS) of measurement results obtained 
% independently, using full enumeration.
% -------------------------------------------------------------------------
% Input
% x                      Measured values 
%                        Dimension: 1 x N
% ux                     Standard uncertainties associated with measured
%                        values
%                        Dimension: 1 x N
% chi2thresh             Threshold chi-squared values for 
%                        0, 1, ..., N - 1 degrees of freedom 
%                        Dimension: N x 1
%
% Output
% NCLS                   Size of LCS
% P                      Indices of measurement results in the LCS
%                        Dimension: NCLS x 1
% yLCS                   Weighted mean of measurement results in the LCS
% uyLCS                  Standard uncertainty associated with weighted
%                        mean
% -------------------------------------------------------------------------
% The measurement result for participant i constitutes a measured value 
% x(i) and an associated standard uncertainty ux(i).
%
% The LCS is identified by the indices P of the measurement results in 
% the subset.
%
% The weighted mean (WM) y and the associated standard uncertainty uy 
% are determined for the LCS.  
%
% The computation is based on considering all subsets of size N, N - 1,
% ... of the measurement results until the largest subset consistent 
% with its WM is obtained.  Of all subsets of that size, that having
% the smallest observed chi-squared value is taken.
% -------------------------------------------------------------------------

%
% Set number of participating measurements and check input data.
%
  N = length(x);
  if N < 1
      error('x must have positive dimension')
  end
  if length(ux) ~= N
      error('x and ux must have identical dimensions')
  end
  if any(ux <= 0)
      error('Elements of ux must be positive')
  end
%
% Initialize size of LCS.
%
  NLCS = 0;
%
% Initialize marker for consistency.
%
  consistent = false;
%
% Consider subsets of successively decreasing size.
%
  for r = N : - 1: 2
%
% Initialize threshold value for observed chi-squared.
%
      chi2LCS = chi2thresh(r);
%
% Set the number of subsets of size r from the N considered.
%
      M = factorial(N)/(factorial(r)*factorial(N - r));
%
% Initialize the index set for the measurement results to be considered.
%
      P = [1:r-1, r-1];
%
% Count over the choices of r measurement results from the N.
%
      for i = 1 : M
%
% Select the next index set in the sequence.
%
          P = Count3(P, N);
%
% Form WM of the subset and the corresponding TSS function value.
%
          [y, uy] = ComputeWeightedMean(x(P), ux(P));
          chi2obs = norm((x(P) - y)./ux(P))^2;
%
% Consider further this WM only if TSS function value <= critical 
% chi-squared value.
%
          if chi2obs <= chi2thresh(r)
%
% Indicate that consistency has been achieved.
%
              consistent = true;
%
% Accept this WM as the best solution so far if the current consistent 
% subset has a smaller TSS function value than the best previously.
%
              if chi2obs < chi2LCS
%
% Update (1) size of LCS, (2) WM of LCS, (3) associated standard 
% uncertainty, (4) indices of ordered contributions to value of 
% corresponding  TSS function, and (5) the value in (4).
%
                  NLCS    = r;
                  yLCS    = y;
                  uyLCS   = uy;
                  PLCS    = P;
                  chi2LCS = chi2obs;
              end
          end
      end
%
% Finish with required LCS if consistent.
%
      if consistent
          break
      end
  end
  if NLCS == 0
%
% Indicate that no LCS exists for the measurement results.
%
      P = (1:N)';
      yLCS = [];
      uyLCS = [];
  else
      P = PLCS;
  end
%
% -------------------------------------------------------------------------
% End LCSFullEnumerate.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011

% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------