function Sens =  LinearCalibration(workbookpath, sensorworksheet, MeasIn)
% -------------------------------------------------------------------------
% LinearCalibration.m
%
% Read sensor information from Excel worksheet, generate sensor responses 
% to measurand based on a linear calibration for sensors allowing for 
% faulty sensors and missing data, calculate estimates of measurand and 
% associated standard uncertainties and store in structure. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% sensorworksheet        Worksheet containing sensor data 
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%
% Output
% Sens                   Structure defining estimates of measurand from 
%                        sensor responses:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  measurand estimates
%                                  Dimension: m2 x nS
% -------------------------------------------------------------------------

% Display sensor response generation method and name of worksheet. 
%
  fprintf('\nType of sensor behaviour: Linear calibration \n');
  fprintf(['Worksheet: %', num2str(length(sensorworksheet)), 's \n'], sensorworksheet); 
% 
% Read sensor information from worksheet. 
%
  sensSetUp = xlsread(workbookpath, sensorworksheet, 'B9:B12');
  nS = sensSetUp(1); % number of sensors 
  f2 = sensSetUp(2); % sampling frequency 
  QI = 0;
  if numel(sensSetUp) > 2
      QI = 1;
      nB = sensSetUp(3); % number of bits (quantization)
      s  = sensSetUp(4); % peak value for saturation (quantization)
  end
  rangeStr = ['D9:I', num2str(8+nS)];
  sensData = xlsread(workbookpath, sensorworksheet, rangeStr);
  AB       = sensData(:,1:2); % offset and gain coefficients 
  urAB     = sensData(:,3:4); % relative standard uncertainties associated with coefficients 
  CovrAB   = sensData(:,5);   % relative covariances associated with coefficients 
  delta    = sensData(:,6);   % additive noise levels 
  nF = xlsread(workbookpath, sensorworksheet, 'L9');
  if nF > 0
      rangeStr  = ['M9:T', num2str(8+nF)];
      sensDataF = xlsread(workbookpath, sensorworksheet, rangeStr);
      IndF      = sensDataF(:,1);   
      CD        = sensDataF(:,2:3); 
      urCD      = sensDataF(:,4:5); 
      CovrCD    = sensDataF(:,6);   
      T12F      = sensDataF(:,7:8); 
  end
  nM = xlsread(workbookpath, sensorworksheet, 'W9');
  if nM > 0
      nD        = xlsread(workbookpath, sensorworksheet, 'W10');
      rangeStr  = ['X9:AA', num2str(8+nM)];
      sensDataM = xlsread(workbookpath, sensorworksheet, rangeStr);
      IndM      = sensDataM(:,1); 
      pM        = sensDataM(:,2); 
      T1M       = sensDataM(:,3); 
      T2M       = sensDataM(:,4); 
  end
%
% Assign measurand. 
%
  y1 = MeasIn.y;
  t1 = MeasIn.t;   
%
% Draw components describing sensor responses from their associated 
% distributions and generate corresponding sensor response values.
%
  RAB = GenerateRandomDraws('N', 1, 2*nS); 
  RAB = reshape(RAB, nS, 2);
  ABs = zeros(nS, 2); 
  for l = 1:nS
      Vl       = diag((urAB(l,:).*AB(l,:)).^2);
      Vl(1,2)  = CovrAB(l)*prod(AB(l,:));
      Vl(2,1)  = Vl(1,2);
      Ql       = chol(Vl);
      ABs(l,:) = AB(l,:) + RAB(l,:)*Ql;
  end
  m1   = length(t1);
  Ymtx = [ones(m1, 1), y1];
  V1   = Ymtx*ABs';
%  
% Draw components describing sensor responses for faulty sensors from their
% associated distributions and generate corresponding sensor response
% values.
%
  for lF = 1:nF
      isen  = IndF(lF);
      T1    = T12F(lF,1);
      T2    = T12F(lF,2);
      IndT1 = find(t1 >= T1, 1);
      IndT2 = find(t1 <= T2, 1, 'last');
      if ~isempty(IndT1)
          RABF    = GenerateRandomDraws('N', 1, 2);
          VF      = diag((urCD(lF,:).*CD(lF,:)).^2);
          VF(1,2) = CovrCD(lF)*prod(CD(lF,:));
          VF(2,1) = VF(1,2);
          QF      = chol(VF);
          CDs     = CD(lF,:) + RABF'*QF; 
          if isempty(IndT2)
              IndT2 = m1;
          end
          V1(IndT1:IndT2,isen) = Ymtx(IndT1:IndT2,:)*CDs';
      end    
  end  
% 
% Generate and add noise. 
% 
  Rd = GenerateRandomDraws('N', 1, m1*nS);
  Rd = reshape(Rd, m1, nS);
  V1 = V1 + (ones(m1, 1)*delta').*Rd;
  Sens.sigma = delta;
%
% Interpolate sensor response values.
% 
  T11    = t1(1) - (t1(2) - t1(1)); % start time for 'true' measurand 
  t2     = (T11:1/f2:t1(end))';
  t2     = t2(2:end); 
  V2     = interp1(t1, V1, t2);
  Sens.t = t2;
  m2     = length(t2);
%
% Quantize data, if required.
%
  if QI == 1
      V2t = GenerateQuantizedSignal(V2, nB, s);
  else 
      V2t = V2; 
  end
%
% Generate estimates of measurand for each sensor.
%
  A     = AB(:,1); 
  B     = AB(:,2); 
  urA   = urAB(:,1); 
  urB   = urAB(:,2); 
  uA    = A.*urA;
  uB    = B.*urB;
  CovAB = A.*B.*CovrAB;
  Av    = ones(m2, 1)*A'; 
  Bv    = ones(m2, 1)*B';
  Y     = (V2t - Av)./Bv; 
%
% Calculate uncertainties associated with measurand estimates for each 
% sensor.
%
  U = zeros(m2, nS);
  for l = 1:nS
      dA = -1/B(l);
      rB = dA^2;
      for i2 = 1:m2
          dB      = -(V2t(i2,l) - A(l))*rB;
          uij     = rB*delta(l)^2 + dA^2*uA(l)^2 + dB^2*uB(l)^2 + 2*dA*dB*CovAB(l);
          U(i2,l) = sqrt(uij);
      end
  end
%
% Remove missing data points.
%
  for lM = 1:nM
      isen  = IndM(lM);
      IndT1 = ceil((T1M(lM) - t1(1))*f2) + 1;
      IndT2 = floor((T2M(lM) - t1(1))*f2) + 1;
      if IndT1 <= m2
          range_md = IndT2 - IndT1 + 1;
          range_mp = floor(range_md/nD);
          if range_md > 0
              n_md = round(pM(lM)*range_mp/100); % number of missing data packets
              Rmp  = GenerateRandomDraws('R', 1, range_mp);
              [X, I_mp] = sort(Rmp); %#ok<ASGLU>
              for mp = 1:n_md
                  I_md = IndT1 + (I_mp(mp) - 1)*nD:IndT1 + I_mp(mp)*nD - 1; 
                  Y(I_md,isen) = NaN;
                  U(I_md,isen) = NaN;
              end
          end
      end   
  end
%
% Store estimates data in structure.
%
  Sens.Y  = Y;
  Sens.uY = U;
%   
% -------------------------------------------------------------------------
% End LinearCalibration.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------