function Sens1 =  LinearCalibrationInterdependenceCov(workbookpath, sensorworksheet, Meas1In, Meas2In, Meas2Out)
% -------------------------------------------------------------------------
% LinearCalibrationInterdependenceCov.m
%
% Generate sensor responses to measurand based on a linear calibration for 
% sensors with dependence on more than one physical quantity. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% sensorworksheet        Worksheet containing sensor data 
% Meas1In                Structure defining true principal measurand:
%                        t         Time values
%                                  Dimension: m11 x 1
%                        y         Measurand values 
%                                  Dimension: m11 x 1
% Meas2In                Structure defining true subsidiary measurand:
%                        t         Time values
%                                  Dimension: m21 x 1
%                        y         Measurand values
%                                  Dimension: m21 x 1
% Meas2Out               Structure defining estimates of subsidiary 
%                        measurand:
%                        t         Time values
%                                  Dimension: m22 x 1
%                        y         Measurand estimates
%                                  Dimension: m22 x 1
%                        uy        Standard uncertainties associated with
%                                  measurand estimates
%                                  Dimension: m22 x 1
%
% Output
% Sens1                  Structure defining estimates of measurand from 
%                        sensor responses:
%                        t         Time values
%                                  Dimension: m12 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m12 x nS
%                        VY        Covariance matrices associated with 
%                                  measurand estimates (at each time value)
%                                  Dimension: m12 x 1 cell (of nS x nS
%                                             arrays)
%                        uY        Standard uncertainties associated with 
%                                  measurand estimates
%                                  Dimension: m12 x nS
% -------------------------------------------------------------------------

% Display sensor response generation method and name of worksheet. 
%
  fprintf('\nType of sensor behaviour: Linear calibration with interdependence \n');
  fprintf(['Worksheet: %', num2str(length(sensorworksheet)), 's \n'], sensorworksheet); 
% 
% Read sensor information from worksheet.
%
  sensSetUp = xlsread(workbookpath, sensorworksheet, 'B9:B12');
  nS = sensSetUp(1); % number of sensors
  f2 = sensSetUp(2); % sampling frequency
  QI = 0;
  if numel(sensSetUp) > 2
      QI = 1;
      nB = sensSetUp(3); % number of bits (quantization)
      s  = sensSetUp(4); % peak value for saturation (quantization)
  end
  rangeStr = ['D9:M', num2str(8+nS)];
  sensData = xlsread(workbookpath, sensorworksheet, rangeStr);
  ABC      = sensData(:,1:3);
  urABC    = sensData(:,4:6);
  CovrABC  = sensData(:,7:9);
  delta    = sensData(:,10);
  nF = xlsread(workbookpath, sensorworksheet, 'P9');
  if nF > 0
      rangeStr  = ['Q9:AB', num2str(8+nF)];
      sensDataF = xlsread(workbookpath, sensorworksheet, rangeStr);
      IndF      = sensDataF(:,1);
      DEF       = sensDataF(:,2:4);
      urDEF     = sensDataF(:,5:7);
      CovrDEF   = sensDataF(:,8:10);
      T12F      = sensDataF(:,11:12);
  end
  nM = xlsread(workbookpath, sensorworksheet, 'AE9');
  if nM > 0
      nD        = xlsread(workbookpath, sensorworksheet, 'AE10');
      rangeStr  = ['AF9:AI', num2str(8+nM)];
      SensDataM = xlsread(workbookpath, sensorworksheet, rangeStr);
      IndM      = SensDataM(:,1);
      pM        = SensDataM(:,2);
      T1M       = SensDataM(:,3);
      T2M       = SensDataM(:,4);
  end
%
% Assign true principal measurand. 
%
  y11 = Meas1In.y;
  t11 = Meas1In.t;  
%
% Assign true subsidiary measurand. 
%
  y21 = Meas2In.y;
  t21 = Meas2In.t;
%
% Interpolate subsidiary measurand values. 
%
  if (length(t11)~= length(t21)) || (sum(t11 == t21) < length(t11))
      y21  = interp1(t21, y21, t11);    
  end
%  
% Draw components describing sensor responses from their associated 
% distributions.
%
  RABC = GenerateRandomDraws('N', 1, 3*nS);
  RABC = reshape(RABC, nS, 3);
  ABCs  = zeros(nS, 3);
  for l = 1:nS
      uABC       = urABC(l,:).*ABC(l,:);
      CovABC     = CovrABC(l,:).* ...
          [prod(ABC(l,1:2)), prod(ABC(l,[1, 3])), prod(ABC(l,2:3))];
      Vl        = diag(uABC.^2);
      Vl(1,2:3) = CovABC(1:2);
      Vl(2:3,1) = Vl(1,2:3);
      Vl(2,3)   = CovABC(3);
      Vl(3,2)   = Vl(2,3);
      Q         = chol(Vl);
      ABCs(l,:) = ABC(l,:) + RABC(l,:)*Q;
  end
% 
% Generate sensor response to principal and subsidiary measurands.
% 
  m11  = length(y11);
  Ymtx = [ones(m11, 1), y21, y11];
  V1   = Ymtx*ABCs';
%  
% Draw components describing sensor responses for faulty sensors from their
% associated distributions and generate corresponding sensor response
% values. 
%
  for lF = 1:nF
      isen  = IndF(lF);
      T1    = T12F(lF,1);
      T2    = T12F(lF,2);
      IndT1 = find(t11 >= T1, 1);
      IndT2 = find(t11 <= T2, 1, 'last');
      if ~isempty(IndT1)
          RDEFF     = GenerateRandomDraws('N', 1, 3);
          uDEF      = urDEF(lF,:).*DEF(lF,:);
          CovDEF    = CovrDEF(lF,:).*[prod(DEF(lF,1:2)), ...
              prod(DEF(lF,[1, 3])), prod(DEF(lF,2:3))];
          VF        = diag(uDEF.^2);
          VF(1,2:3) = CovDEF(1:2);
          VF(2:3,1) = VF(1,2:3);
          VF(2,3)   = CovDEF(3);
          VF(3,2)   = VF(2,3);
          QF        = chol(VF);
          DEFs      = DEF(lF,:) + RDEFF'*QF;
          if isempty(IndT2)
              IndT2 = m11;
          end
          V1(IndT1:IndT2, isen) = Ymtx(IndT1:IndT2,:)*DEFs';
      end    
  end 
% 
% Generate and add noise. 
% 
  Rd = GenerateRandomDraws('N', 1, m11*nS);
  Rd = reshape(Rd, m11, nS);
  V1  = V1 + (ones(m11, 1)*delta').*Rd;
  Sens1.sigma = delta;
%
% Interpolate sensor response values.
%
  T11     = t11(1) - (t11(2) - t11(1)); % start time for 'true' measurand  
  t12     = (T11:1/f2:t11(end))';
  t12     = t12(2:end); 
  V2      = interp1(t11, V1, t12);
  Sens1.t = t12;
  m12     = length(t12);
%
% Quantize data, if required.
%
  if QI == 1
      V2t = GenerateQuantizedSignal(V2, nB, s);
  else 
      V2t = V2; 
  end
%
% Generate measurand indication for each sensor.
%
   t22  = Meas2Out.t;
   y22  = Meas2Out.y;
   uy22 = Meas2Out.uy;
%
% If sampling times for principal and subsidiary measurands are not the
% same, interpolate estimates of subsidiary measurand and associated 
% uncertainties (assuming continuity of uncertainty).
%
   y22  = interp1(t22, y22, t12);
   uy22 = interp1(t22, uy22, t12);
   A  = ABC(:,1); 
   B  = ABC(:,2);
   C  = ABC(:,3);
   uA  = A.*urABC(:,1);
   uB  = B.*urABC(:,2);
   uC  = C.*urABC(:,3);
   CovAB = A.*B.*CovrABC(:,1);
   CovAC = A.*C.*CovrABC(:,2);
   CovBC = B.*C.*CovrABC(:,3);
   Av      = ones(m12,1)*A';
   Bv      = y22*B';
   Cv      = ones(m12,1)*C';
   Y       = (V2t - Av - Bv)./Cv; 
%
% Calculate covariance matrices associated with measurand estimates for
% each sensor.
%
   VY = cell(m12, 1);
   uY = zeros(m12, nS);
   dV = 1./C;
   dA = -dV;
   rC = dV.^2;
   dx = -B.*dV;
   for i12 = 1:m12
       Vx = dx*dx'*uy22(i12)^2;
       dB = -y22(i12)*dV;
       dC = -(V2t(i12,:)' - A - B*y22(i12)).*rC;
       uSens = rC.*delta.^2 + (dA.*uA).^2 + (dB.*uB).^2 +...
           (dC.*uC).^2 + 2*dA.*dB.*CovAB + 2*dA.*dC.*CovAC +...
           2*dB.*dC.*CovBC;
       VY{i12} = diag(uSens) + Vx;
       uY(i12,:) = sqrt(diag(VY{i12}));
   end
%
% Remove missing data points.
%
   for imd = 1:nM
       isen  = IndM(imd);
       st_n  = ceil((T1M(imd) - t11(1))*f2) + 1;
       end_n = floor((T2M(imd) - t11(1))*f2) + 1;
       if st_n <= m12
           range_md = end_n - st_n + 1;
           range_mp = floor(range_md/nD);
           if range_md > 0
               n_md = round(pM(imd)*range_mp/100); % number of missing data packets
               Rmp  = GenerateRandomDraws('R', 1, range_mp);
               [X, I_mp] = sort(Rmp); %#ok<ASGLU>
               for mp = 1:n_md
                   I_md = st_n + (I_mp(mp) - 1)*nD:st_n + I_mp(mp)*nD - 1; 
                   Y(I_md,isen) = NaN;
                   for dp = 1:nD
                       VYdp = VY{I_md(dp)};
                       VYdp(isen,:) = NaN;
                       VYdp(:,isen) = NaN;
                       VY{I_md(dp)} = VYdp;
                   end
               end
           end
       end   
   end
%
% Store sensor responses in structure.
%
   Sens1.Y  = Y;
   Sens1.VY = VY;
   Sens1.uY = uY;
%
% -------------------------------------------------------------------------
% End LinearCalibrationInterdependenceCov.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------