function MeasOut = MaxCliqueDataFusion(Sens)
% -------------------------------------------------------------------------
% MaxCliqueDataFusion.m
%
% Form estimates of the measurand and associated uncertainties from sensor 
% estimates and associated uncertainties using the maximum clique data 
% fusion method.
% -------------------------------------------------------------------------
% Input
% Sens                   Structure defining estimates of measurand from 
%                        sensor responses:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  measurand estimates
%                                  Dimension: m2 x nS
% 
% Output
% MeasOut                Structure defining estimate of measurand:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        y         Measurand estimates
%                                  Dimension: m2 x 1
%                        uy        Standard uncertainties associated with
%                                  estimates
%                                  Dimension: m2 x 1
%                        indices   Indices of consistent sensors
%                                  Dimension: m2 x 1
% -------------------------------------------------------------------------

% Display data fusion method.
%
  fprintf('\nData fusion method: Maximum clique \n');
%  
% Initialize variables.
%
  Y       = Sens.Y;
  uY      = Sens.uY;
  t       = Sens.t;
  m2      = size(Y,1);
  y       = zeros(m2,1);
  uy      = zeros(m2,1);
  indices = cell(m2,1);
  dispInd = 0; % indicator for displaying consistent subset warning
  kcov    = 1.96 ; % coverage factor for 95 % level of confidence
%
% Estimate measurand.
%
  for it = 1:m2
      YI  = find(~isnan(Y(it,:)));
      if isempty(YI)
          NLCS = 0;
      else
          n   = length(Y(it,YI));
          x   = Y(it,YI);
          ux  = uY(it,YI);
          dm  = ComputeMoffatDis(x, ux);
%
% Determine largest consistent subset by exhaustive search; satisfy 
% condition dm(i,j) < k.
%
          consistent = false;
%
% Consider subsets of successively decreasing size.
%
          ILCS = [];
          for r = n:-1:2            
%
% Set the number of subsets of size r from the n considered.
%
              M = factorial(n)/(factorial(r)*factorial(n - r));
%
% Initialize the index set for the measurement results to be considered.
%
              PI = [1:r-1, r-1];
%
% Count over the choices of r measurement results from the n.
%
              LCSS = [];
              for i = 1 : M
%
% Select the next index set in the sequence.
%
                  PI = Count3(PI, n);
%
% Check for consistency.
%
                  OK = (max(max(dm(PI,PI))) < kcov);        
                  if OK % consistent subset found 
                      LCSS       = [LCSS; PI]; %#ok<AGROW>
                      consistent = true;
                      ILCS       = PI;
                  end
              end
%    
% Finish with required LCS if consistent.
%
              if consistent
                  LCS = LCSS(1,:); % first found subset
%                  
% Compute intersect of all subsets.
%
                  ns  = size(LCSS,1);
                  UNS = [];
                  for is = 2:ns
                      LCS = intersect(LCSS(is,:), LCS);
                      UNS = union(UNS, LCSS(is,:));
                  end    
                  if ~isempty(LCS)
                      ILCS = LCS;
                  else  % intersect of all largest subsets is empty 
                      ILCS = LCSS(1,:); % use first subset            
                  end    
                  break
              end
          end
%          
% Compute augmented (multiplied) uncertainty by largest Moffat distance
% for each measuremnt not in the LCS.
%
          if consistent
              PF      = 1:n;
              badSet = setdiff(PF, ILCS);
              for ib = badSet
%              
% Compute maximum Moffat distance to any in the LCS.
%
                  dis    = max(dm(ib,ILCS));  
                  ux(ib) = ux(ib)*max(dis, 1);  
              end
%          
% Compute combined best estimate.
%
              [ywm, uywm] = ComputeWeightedMean(x(PF), ux(PF));
              PI          = ILCS; 
              NLCS        = length(PF);
              yLCS        = ywm; 
              uyLCS       = uywm;
              P           = YI(PI);
          else
              NLCS = 0;
          end
      end
      if  NLCS > 0
          y(it)       = yLCS; 
          uy(it)      = uyLCS;
          indices(it) = {P};
      end
%  
% Check for case no consistent subset.
%
      if NLCS == 0
          if dispInd == 0
              txt = ['WARNING: No consistent subset found at t = ',num2str(t(it)), 's']; 
              disp(txt);
              disp('Warning will not show again for any subsequent times with no consistent subset');
              dispInd = 1;
          end
          if (it > 1)
%          
% Take measurand estimate from previous time point.
%
              y(it)       = y(it-1);
              uy(it)      = uy(it-1);
              indices(it) = {[]}; % Empty set
          elseif ~isempty(YI)
%          
% Use the output from the first functioning sensor.
%
              y(it)       = Y(it,YI(1));
              uy(it)      = uY(it,YI(1));
              indices(it) = {YI(1)};
          else
              y(it)       = NaN;
              uy(it)      = NaN;
              indices(it) = {[]}; % Empty set
          end
      end
  end
% 
% Store estimates data in structure.
% 
  MeasOut.t       = t;
  MeasOut.y       = y; 
  MeasOut.uy      = uy; 
  MeasOut.indices = indices; 
end    
%
% -------------------------------------------------------------------------
% End MaxCliqueDataFusion.m
% -------------------------------------------------------------------------
%
  function dm = ComputeMoffatDis(dx, udx)
% -------------------------------------------------------------------------
% ComputeMoffatDis.m
%
% Compute pairwise Moffat distances. 
% -------------------------------------------------------------------------
% Input
% dx                     Estimates
%                        Dimension: N x 1
% udx                    Standard uncertainties associated with estimates 
%                        Dimension: N x 1
% 
% Output
% dm                     Moffat distances
%                        Dimension: N x N
% -------------------------------------------------------------------------

  N  = length(dx);
  dm = zeros(N,N);
  for ix1 = 1:N-1
      for ix2 = ix1+1:N
          dm(ix1,ix2) = abs(dx(ix1) - dx(ix2))/sqrt(udx(ix1)^2 + udx(ix2)^2);
          dm(ix2,ix1) = dm(ix1,ix2);
      end
  end
end
%
% -------------------------------------------------------------------------
% End ComputeMoffatDis.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------