function MeasIn = PiecewiseLinear(workbookpath, measurandworksheet)
% -------------------------------------------------------------------------
% PiecewiseLinear.m
%
% Read information describing a piecewise linear measurand from Excel 
% worksheet, generate measurand and store in structure. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% measurandworksheet     Worksheet containing measurand data 
%
% Output
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%                        name      Measurand name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
% -------------------------------------------------------------------------

% Display measurand simulation method and name of worksheet. 
%
  fprintf('\nType of measurand simulation: Piecewise linear \n');
  fprintf(['Worksheet: %', num2str(length(measurandworksheet)), 's \n'], ... 
      measurandworksheet); 
%  
% Read measurand information from worksheet. 
%
  [num, qName] = xlsread(workbookpath, measurandworksheet, 'B4'); %#ok<ASGLU>
  [num, qUnit] = xlsread(workbookpath, measurandworksheet, 'B5'); %#ok<ASGLU>
  if isempty(qUnit)
      qUnit = {'unit'};
  end
  if isempty(qName)
      qName = {'Measurand'};
      yStr = ['Quantity/', qUnit{1}]; % string for plot axis
  else
      yStr = [qName{1}, '/', qUnit{1}];
  end  
  measSetUp  = xlsread(workbookpath, measurandworksheet, 'B9:B11');
  f1         = measSetUp(1); % sampling frequency
  D          = measSetUp(2); % signal duration
  nL         = measSetUp(3); % number of segments
  rangeStr   = ['C9:H', num2str(8 + nL)];
  measData   = xlsread(workbookpath, measurandworksheet, rangeStr); 
  T          = measData(:,1);     % start times 
  alphabeta  = measData(:,[2,4]); % estimates of 'intercept' and 'gradient' coefficients 
  ualphabeta = measData(:,[3,5]); % relative standard uncertainties associated with estimates of coefficients 
  sigma      = measData(:,6);     % additive noise levels
%
% Generate measurand and store in structure.
%
  MeasIn = GeneratePiecewiseLinear(f1, D, T, alphabeta, ualphabeta, sigma);
  MeasIn.name = qName;
  MeasIn.yStr = yStr;
%  
% -------------------------------------------------------------------------  
% End PiecewiseLinear.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------