function PlotObsMeasOut(MeasIn, Obs, Sens, MeasOut)
% -------------------------------------------------------------------------
% PlotObsMeasOut.m
%
% Generate plots of:
% - Observable output signal. 
% - Sensor estimates of observable output signal, sensor estimates of 
% observable output signal plus associated standard uncertainties. 
% - True measurand, combined estimates of measurand, combined estimates of 
% measurand plus associated standard uncertainties, combined estimates of 
% measurand minus associated standard uncertainties. 
% - Standard uncertainties associated with combined estimates of measurand.
% - Status of sensors. 
% -------------------------------------------------------------------------
% Input
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%                        name      Measurand name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
%                                  Dimension: string
% Obs                    Structure defining observed system response:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         System response values 
%                                  Dimension: m1 x 1
%                        name      Observation name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
% Sens                   Structure defining estimates of observable output 
%                        signal from sensor responses: 
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Observable output signal estimates
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  observable output signal estimates
%                                  Dimension: m2 x nS
% MeasOut                Structure defining estimate of measurand:
%                        y         Measurand estimates
%                                  Dimension: m2 x 1
%                        uy        Standard uncertainties associated with
%                                  measurand estimates
%                                  Dimension: m2 x 1
%                        indices   Indices of consistent sensors
%                                  Dimension: m2 x 1
% -------------------------------------------------------------------------

% Define data. 
%
  t1    = MeasIn.t;
  y1    = MeasIn.y;
  yStr  = MeasIn.yStr;
  yObs  = Obs.y;
  Y2    = Sens.Y;
  uY2   = Sens.uY;
  y2    = Sens.t;
  nS    = size(Y2, 2);
  m2    = length(y2);
  yE    = MeasOut.y; 
  uyE   = MeasOut.uy;
%
% Plot observable output signal. 
%
  figure
  plot(t1, yObs);
  ax    = axis; 
  ax(2) = max(t1); 
  axis(ax);
  xlabel('Time/s')
  ylabel(Obs.yStr)
  if strcmp(Obs.name{1}, 'Observation')
      titleStr = 'Observable output signal';
  else
      titleStr = ['Observable ', lower(Obs.name{1})];
  end
  title(titleStr)
%  
% Plot sensor estimates of observable output signal and associated 
% uncertainties. 
%
  figure; hold on
  strlegend = [];
  colour    = ['b', 'g', 'r', 'c', 'm', 'k', 'y', 'w'];
  colL      = ceil(nS/8);
  colour1   = []; 
  for ic = 1:colL
      colour1 = strcat(colour1, colour);
  end
%
  for is = 1:nS
      plot(y2, Y2(:,is), colour1(is), y2, Y2(:,is) + uY2(:,is), [colour1(is),':']);
      str       = ['y_{', num2str(is), '}           '];
      strlegend = [strlegend; str]; %#ok<AGROW>
      str2      = ['y_{', num2str(is), '}'];
      stru      = [str2,' + u(y_{', num2str(is), '})'];
      strlegend = [strlegend; stru]; %#ok<AGROW>
  end
  legend(strlegend, 'Location', 'EastOutside');
  xlabel('Time/s');
  ylabel(Obs.yStr);
  if strcmp(Obs.name{1}, 'Observation')
      titleStr = 'Sensor estimates of observable output signal and associated uncertainties';
  else
      titleStr = [Obs.name{1}, ' sensor outputs and associated uncertainties'];
  end
  title(titleStr);
  ax    = axis; 
  ax(2) = y2(m2); 
  axis(ax);
  hold off
%   
% Plot 'true' and estimated measurand. 
%
  figure
  plot(t1, y1, y2, yE, y2, yE + 2*uyE, y2, yE - 2*uyE);
  legend('True', 'Estimated', 'Est y + 2u(y)','Est y - 2u(y)','Location','EastOutside');
  xlabel('Time/s')
  ylabel(yStr);
  titleStr = ['True and estimated ', lower(MeasIn.name{1})];
  title(titleStr);
  ax    = axis; 
  ax(2) = y2(m2); 
  axis(ax);
%   
% Plot uncertainty associated with estimated measurand. 
%
  figure
  plot(y2, uyE);
  xlabel('Time/s')
  ylabel(yStr);
  titleStr = ['Standard uncertainties associated with ', lower(MeasIn.name{1}), ' estimates'];
  title(titleStr);
  ax    = axis; 
  ax(2) = y2(m2); 
  axis(ax);
%
% Plot sensors contained within largest consistent subset.
%
  PlotSensorStatus(Obs.name, MeasOut.indices, y2, nS)
%  
% -------------------------------------------------------------------------
% End PlotObsMeasOut.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------