function  PlotSensorStatus(measName, indices, t, nS)
% -------------------------------------------------------------------------
% PlotSensorStatus.m
%
% Display sensor status, i.e., indicate if sensor is in largest consistent 
% subset (LCS) or not. 
% -------------------------------------------------------------------------
% Input
% measName               Measurand name
%                        Dimension: 1 x 1 cell
% indices                Indices of consistent sensors
%                        Dimension: m2 x 1
% t                      Time values
%                        Dimension: m2 x 1
% nS                     Number of sensors
% -------------------------------------------------------------------------

% Assign data. 
%
  m2 = length(t);
  sensStatus = zeros(nS + 1, m2); % add extra column for Kalman Filter data fusion method 
  for it = 1:m2
      idxs = indices{it} + 1;
      sensStatus(idxs,it) = 1;
  end    
%
% Remove column if Kalman Filter data fusion method is not used. 
%
  if sum(sensStatus(1,:)) == 0
      sensStatus(1,:) = [];
      flagKF = 0;
  else
      flagKF = 1;
  end
%
% Plot sensors in the LCS.
%
  ys = ones(nS + flagKF, 1);
  figure
  lw = 2;
  color2 = {'cyan', 'black'};
  for is = 1:nS + flagKF
      ys(is) = is;
      icolor = color2{1};
      line([t(1) t(m2)], [ys(is), ys(is)], 'LineWidth', lw, 'Color', icolor);
      hold on
  end
%
% Plot sensors not in the LCS.
%
  for is = 1:nS + flagKF
      ys(is) = is;
      icolor = color2{2};
      IDX = find(sensStatus(is,:) == 0); % time points at which not in LCS
      for iit = 1:length(IDX) % for each time point...
          it = IDX(iit); 
          if it < m2
              plot(t(it), ys(is), '*', 'MarkerSize', 5, 'Color', icolor);     
              hold on
          end    
      end    
  end
%
% Label and annotate plot.
%
  xlabel('Time/s')
  ylabel('Sensors')
  if strcmp(measName{1}, 'Measurand')
      titleStr = 'Sensor status';
  else
      titleStr = [measName{1}, ' sensor status'];
  end
  title(titleStr);
  ax = axis; 
  ax(2) = t(m2); % x-axis upper limit 
  ax(3)=0; ax(4) = nS + flagKF + 1; % y-axis limits
  axis(ax);
  if flagKF == 0
      set(gca, 'ytick', 1:nS);
  else
      set(gca, 'ytick', 1:nS + flagKF, 'yticklabel', {'KF Model', 1:nS});
  end
  v = axis;
  x = (v(1) + v(2))/2 ;
  width = v(2)/50;
  y = nS + flagKF + 0.5;
  line(x, y, 'Color', color2{2}, 'LineStyle', 'none', 'Marker', '*');
  text(x + width, y, 'Out of LCS');
%  
% -------------------------------------------------------------------------
% End PlotSensorStatus.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------