function [xp, Sp, indices] = RunKalmanFilter(xm, Sm, Phi, G, QL, z, H, RL, Sp0)
% -------------------------------------------------------------------------
% RunKalmanFilter.m
%
% Form estimate of the measurand and its associated uncertainty using the 
% Kalman filter data fusion method. Measurement update is applied using the
% largest consistent subset of observations and model prediction, 
% determined by full enumeration. It is assumed that observations are all 
% of the same quantity. 
% -------------------------------------------------------------------------
% Input
% xm                     State vector before update
%                        Dimension: n x 1
% Sm                     Square root factor of state covariance matrix
%                        before update
%                        Dimension: n x n
% Phi                    State propagation matrix
%                        Dimension: n x n
% G                      G and QL define the process covariance matrix Q:
%                        Q = G*QL*QL'*G'
%                        Dimension: n x m
% QL                     Dimension: m x m
% z                      Vector of observations
%                        Dimension: nS x 1
% H                      Design matrix
%                        Dimension: nS x n
% RL                     RL defines the process covariance matrix R:
%                        R = RL*RL'
%                        Dimension: nS x nS
% Sp0                    Inflated square root factor of state covariance
%                        matrix if model prediction not in LCS
%                        Dimension: n x n
%
% Output
% xp                     State vector after update
%                        Dimension: n x 1
% Sp                     Square root factor of state covariance matrix
%                        after update
%                        Dimension: n x n
% indices                Indices of consistent sensors and model
%                        Dimension: 1 x nP
% -------------------------------------------------------------------------
% Update state and covariance matrix based on process model.
%
  [xp, Sp] = UpdateProcessSRCF(xm, Sm, Phi, G, QL);
%
% Search for dropouts (identified by NaNs) in sensor values.
%
  Q  = find(~isnan(z));
  z  = z(Q);
  H  = H(Q,:);
  RL = RL(Q, Q);
% 
% Identify largest consistent subset of model prediction and sensor 
% measurements. 
% 
  x1  = H(1,:)*xp; 
  x   = [x1; z]; 
  Vx  = Sp*Sp'; 
  ux1 = sqrt(diag(Vx(1, 1))); 
  Vz  = RL*RL'; 
  uz  = sqrt(diag(Vz)); 
  ux  = [ux1; uz];
  N   = length(x); 
  chi2thresh = [0; ChiSquared(1:N-1)];  
  [NLCS, P] = LCSFullEnumerate(x, ux, chi2thresh); %#ok<ASGLU>
  nP = length(P);
%
% Remove inconsistent sensors and inflate uncertainty associated with model
% prediction if model inconsistent.
%
  QM = [0; Q];
  indices = sort(QM(P));
  if min(P) == 1 % model prediction belongs to LCS
      if nP < N  % at least one sensor to be dropped 
          z  = z(P(2:end)-1); 
          H  = H(P(2:end)-1,:); 
          RL = RL(P(2:end)-1, P(2:end)-1); 
      end 
  else              % model prediction does not belong to LCS
      if nP < N - 1 % at least one sensor to be dropped 
          z  = z(P(1:end)-1); 
          H  = H(P(1:end)-1, :); 
          RL = RL(P(1:end)-1, P(1:end)-1); 
      end
      Sp = Sp0; % inflate uncertainty associated with model prediction 
  end
%  
% Update state and covariance matrix based on measurement model.
%
  [xp, Sp] = UpdateMeasSRCF(xp, Sp, z, H, RL);
%
% -------------------------------------------------------------------------
% End RunKalmanFilter.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------