function [Obs, Phi] = SecondOrderSystemObs(workbookpath, observationworksheet, MeasIn)
% -------------------------------------------------------------------------
% SecondOrderSystemsObs.m
%
% Read information describing a second order system response to a  
% disturbance from Excel worksheet, generate observed system response and 
% state propagation matrices and store in structures. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% observationworksheet   Worksheet containing observation data 
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
%
% Output
% Obs                    Structure defining observed system response:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         System response values 
%                                  Dimension: m1 x 1
%                        name      Observation name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
% Phi                    Structure defining state propagation matrices:
%                        A         First state propagation matrix
%                                  Dimension: 2 x 2
%                        B         Second state propagation matrix
%                                  Dimension: 2 x 1
%                        C         Third state propagation matrix
%                                  Dimension: 1 x 2
%                        D         Fourth state propagation matrix
%                                  Dimension: 1 x 1
% -------------------------------------------------------------------------

% Display observation simulation method and name of worksheet.
%
  fprintf('\nType of observation simulation: Second order system \n');
  fprintf(['Worksheet: %', num2str(length(observationworksheet)), 's \n'], ... 
      observationworksheet); 
%  
% Read observation information from worksheet. 
%
  [num, qName] = xlsread(workbookpath, observationworksheet, 'B4'); %#ok<ASGLU>
  [num, qUnit] = xlsread(workbookpath, observationworksheet, 'B5'); %#ok<ASGLU>
  if isempty(qUnit)
      qUnit = {'unit'};
  end
  if isempty(qName)
      qName = {'Observation'};
      yStr = ['Quantity/', qUnit{1}]; % string for plot axis
  else
      yStr = [qName{1}, '/', qUnit{1}];
  end
  yData = xlsread(workbookpath, observationworksheet, 'C9:D9');
%
% Generate structures containing observed system response and state
% propagation matrices.
%
  t = MeasIn.t;
  y = MeasIn.y;
  [Obs, Phi] = GenerateSystemResponse(t, y, yData);
  Obs.name = qName;
  Obs.yStr = yStr;
%  
% -------------------------------------------------------------------------  
% End SecondOrderSystemObs.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------