function Sens =  SecondOrderSystemSens(workbookpath, sensorworksheet, Obs)
% -------------------------------------------------------------------------
% SecondOrderSystemSens.m
%
% Read sensor information from Excel worksheet, generate sensor responses 
% to measurand based on second order system sensors (assuming no faulty 
% sensor and no missing data), calculate estimates of measurand and 
% associated standard uncertainties and store in structure. 
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% sensorworksheet        Worksheet containing sensor data 
% Obs                    Structure defining observed system response:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         System response values 
%                                  Dimension: m1 x 1
%
% Output
% Sens                   Structure defining estimates of measurand from 
%                        sensor responses:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Measurand estimates 
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  measurand estimates
%                                  Dimension: m2 x nS
% -------------------------------------------------------------------------

% Display sensor response generation method and name of worksheet. 
%
  fprintf('\nType of sensor behaviour: Second order system \n');
  fprintf(['Worksheet: %', num2str(length(sensorworksheet)), 's \n'], ... 
      sensorworksheet); 
% 
% Read sensor information from worksheet.
%
  sensSetUp = xlsread(workbookpath, sensorworksheet, 'B9:B14');
  nS        = sensSetUp(1);
  f2        = sensSetUp(2);
  fRange    = sensSetUp(3:4);
  QI        = 0;
  if numel(sensSetUp) > 4
      QI = 1;
      nB = sensSetUp(5); % number of bits (quantization)
      s  = sensSetUp(6); % peak value for saturation (quantization)
  end
  rangeStr  = ['D9:F', num2str(8+nS)];
  sensData  = xlsread(workbookpath, sensorworksheet, rangeStr);
  param     = sensData(:,1:2);
  delta     = sensData(:,3);
%
% Assign simulated observation.
%
  y = Obs.y;
  t = Obs.t;  
%
% Generate sensor response to input measurand assuming known frequency and 
% Q-factor.
%
  m1 = length(y);
  dt = t(2) - t(1);
  V1  = [];
      
% Compute analogue model of sensors.
%
  fr = param(:, 1); % resonant frequencies 
  Qr = param(:, 2); % Q-factors for damping
  wr = 2*pi*fr;
  dr = -wr./(2*Qr);
  c1 = -dr;
  c0 = wr.^2 + dr.^2;
%
% Express the analogue model as a difference equation model.
%
  b0 = dt^2./(c0*dt^2 + 4*c1*dt + 4);
  b1 = 2*dt^2./(c0*dt^2 + 4*c1*dt + 4);
  b2 = dt^2./(c0*dt^2 + 4*c1*dt + 4);
  a0 = (c0*dt^2 - 4*c1*dt + 4)./(c0*dt^2 + 4*c1*dt + 4);
  a1 = (2*c0*dt^2 - 8)./(c0*dt^2 + 4*c1*dt + 4);
%
% Generate sensor output based on difference equation model.
%
  for l = 1:nS
      vsi = filter([b2(l), b1(l), b0(l)], [1, a1(l), a0(l)], y);
      V1  = cat(2, V1, vsi);
  end
%
% Generate and add noise. 
%
  Rd = GenerateRandomDraws('N', 1, m1*nS);
  Rd = reshape(Rd, m1, nS);
  V1 = V1 + (ones(m1, 1)*delta').*Rd;
%
% Sample data.
%
  duration = t(end);
  t2       = (1/f2:1/f2:duration)';
  V2       = interp1(t, V1, t2);
  m2       = length(t2);
%
% Quantize data, if required.
%
  if QI == 1
      V2t = GenerateQuantizedSignal(V2, nB, s);
  else 
      V2t = V2; 
  end
%
% Generate estimates of measurand for each sensor.
%
  fE = linspace(fRange(1), fRange(2), 25);
  wE = 2*pi*fE;
  g = zeros(1, nS);
  uY = zeros(m2, nS);
%      
% Evaluate amplitude response for an interval of frequencies containing
% those defining the process output.
%  
  for l = 1:nS
      c0i = c0(l);
      c1i = c1(l);
      AA  = (c0i - wE.^2)./((2*c1i*wE).^2 + (c0i - wE.^2).^2);
      BB  = (-2*c1i*wE)./((2*c1i*wE).^2 + (c0i - wE.^2).^2);
      RR  = sqrt(AA.^2 + BB.^2);
%
% Evaluate estimated gain for static inversion.
%
      g(l) = mean(RR);
%
% Calculate uncertainties associated with measurand estimates for each 
% sensor.
%
      uyi     = delta(l)/g(l);
      uY(:,l) = ones(m2, 1)*uyi;
  end
  Y = V2t./(ones(m2, 1)*g);
%
% Store estimates data in structure.
%
   Sens.t  = t2;
   Sens.Y  = Y;
   Sens.uY = uY;
%   
% -------------------------------------------------------------------------
% End SecondOrderSystemSens.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------