function [xp, Sp] = UpdateMeasSRCF(xm, Sm, z, H, RL)
% -------------------------------------------------------------------------
% UpdateMeasSRCF.m
%
% Update state and covariance matrix based on measurement model: 
%     square root covariance filter.
%
% The measurement model is
%    z = H*xp + v, E[v] = 0, V[v] = R = RL*RL'.
% -------------------------------------------------------------------------
% Input
% xm                     State vector before update
%                        Dimension: n x 1
% Sm                     Square root factor of state covariance matrix
%                        before update
%                        Dimension: n x n
% z                      Vector of observations.
%                        Dimension: p x 1 
% H                      Design matrix
%                        Dimension: p x n 
% RL                     RL defines the process covariance matrix R:
%                        R = RL*RL'
%                        Dimension: p x p 
% 
% Output
% xp                     State vector after update (not computed if the
%                        innovation matrix is near-singular)
%                        Dimension: n x 1
% Sp                     Square root factor of state covariance matrix
%                        after update
%                        Dimension: n x n
% -------------------------------------------------------------------------
% B D O Anderson and J B Moore. Optimal Filtering. Prentice-Hall, 
% 1979.
%
% M Vanbegin, P van Dooren and M Verhaegen. Algorithm 675: FORTRAN
% subroutines for computing the square root covariance filter and
% square root information filter in dense or Hessenberg forms.
% ACM Transactions on Mathematical Software, 15, 243-256, 1988.
% -------------------------------------------------------------------------

% Define number n of states and number p of observations.
%
  [p, n] = size(H);
%
% Construct pre-array W.
%
  W = [RL, H*Sm; zeros(n,p), Sm];
%
% Apply triangularization.
% 
% Step 1: eliminate the (1,2) block of W.
%
  for i = 1:p
      [W(i,i), z0, z1] = f06fsf_v(W(i,i), W(i,p+1:p+n));
      [W(i+1:p+n,i), W(i+1:p+n,p+1:p+n)] = ...
          f06fuf_v(z0', z1, W(i+1:p+n,i), W(i+1:p+n,p+1:p+n));
  end
%
% Step 2: triangularize the (2,2) block of W.
%
  for i = 1:n
      [W(p+i,p+i), z0, z1] = f06fsf_v(W(p+i,p+i), W(p+i,p+i+1:p+n));
      [W(p+i+1:p+n,p+i), W(p+i+1:p+n,p+i+1:p+n)] = ...
          f06fuf_v(z0', z1, W(p+i+1:p+n,p+i), W(p+i+1:p+n,p+i+1:p+n));
  end
%
% Evaluate xp (if the gain is computable) and Sp.
%
  G  = W(p+1:p+n,1:p);
  D  = tril(W(1:p,1:p));
  Sp = tril(W(p+1:p+n,p+1:p+n));
  c  = rcond(D);
  if c < eps*p^2
      disp('UpdateMeasSRCF: innovation matrix is singular')
      xp = zeros(n,1);
  else
      K  = G/D;
      xp = xm + K*(z - H*xm);
  end
end 
%
% -------------------------------------------------------------------------
% End UpdateMeasSRCF.m
% -------------------------------------------------------------------------
%
function [alpha, x, z1] = f06fsf_v(alpha, x)
% -------------------------------------------------------------------------
% Modified version of F06FUF.
%
% F06FSF generates details of a Householder reflection, P, such that
%
%    P*(alpha) = (beta),   P'*P = I.
%      (  X  )   (  0 )
%
% P is given in the form
%
%    P = I - (1/z(1))*z*z',
%
% where z is an (n + 1) element vector.
%
% z(1) is returned in z1. If the elements of x are all zero, then 
% z1 is returned as zero and P can be taken to be the unit matrix. 
% Otherwise z1 always lies in the range (1.0, 2.0).
%
% The remaining elements of z are overwritten on x and beta is
% overwritten on alpha.
% -------------------------------------------------------------------------

  n = length(x);
  if n < 1
      z1 = 0;
  else
      scale = norm(x);
      if scale == 0
          z1 = 0;
      else
          if alpha ~= 0
              scale = sqrt(alpha^2 + scale^2);
              if alpha >= 0
                  beta = -abs(scale);
              else
                  beta =  abs(scale);
              end
              z1 = (beta - alpha)/beta;
          else
              beta = -scale;
              z1 = 1;
          end
          x = (-1/beta)*x;
          alpha = beta;
      end
  end
end 
%
% -------------------------------------------------------------------------
% End f06fsf_v.m
% -------------------------------------------------------------------------
%
function [alpha, X] = f06fuf_v(z, z1, alpha, X)
% -------------------------------------------------------------------------
% Matrix version of F06FUF.
%
% F06FUFV performs a Householder reflection given by
%
%    ( alpha1 x1 )   ( alpha1 x1 ) * P
%    ( alpha2 x2 ) = ( alpha2 x2 )
%    (    ...    )   (    ...    )
%
% where the orthogonal matrix P is given in the form
%
%    P = I - (1/z(1))*z*z'.
%
% z(1) must be supplied in z1 and the remaining n elements in z.
% If z1 is zero then P is assumed to be the unit matrix and the
% transformation is skipped, otherwise z1 must be in the range
% (1.0, 2.0). z1 and z will usually be supplied by routine F06FSF.
% -------------------------------------------------------------------------

  if z1 ~= 0
      beta  = z1*alpha + X*z;
      alpha = alpha - beta;
      X     = (-beta/z1)*z' + X;
  end
end 
%
% -------------------------------------------------------------------------
% End f06fuf_v.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------