function WriteObsResults(workbookpath, resultsworksheet, MeasIn, Obs, Sens, MeasOut, seed)
% -------------------------------------------------------------------------
% WriteObsResults.m
%
% Write results from simulation including intermediate observation to Excel 
% worksheet.
% -------------------------------------------------------------------------
% Input
% workbookpath           Path to Excel workbook
% sensorworksheet        Worksheet to which results are to be written
% MeasIn                 Structure defining true measurand:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         Measurand values 
%                                  Dimension: m1 x 1
% Obs                    Structure defining observed system response:
%                        t         Time values
%                                  Dimension: m1 x 1
%                        y         System response values 
%                                  Dimension: m1 x 1
%                        name      Observation name
%                                  Dimension: 1 x 1 cell
%                        yStr      Label for plot axis
% Sens                   Structure defining estimates of observable output 
%                        signal from sensor responses: 
%                        t         Time values
%                                  Dimension: m2 x 1
%                        Y         Observable output signal estimates 
%                                  Dimension: m2 x nS
%                        uY        Standard uncertainties associated with 
%                                  observable output signal estimates
%                                  Dimension: m2 x nS
% MeasOut                Structure defining estimates of measurand:
%                        t         Time values
%                                  Dimension: m2 x 1
%                        y         Measurand estimates
%                                  Dimension: m2 x 1
%                        uy        Standard uncertainties associated with
%                                  measurand estimates
%                                  Dimension: m2 x 1
%                        indices   Indices of consistent sensors
%                                  Dimension: m2 x 1
% seed                   Seed values used for random number generation
%                                  Dimension: 1 x 4
% -------------------------------------------------------------------------

% Suppress MATLAB warning.
%
  warning off MATLAB:xlswrite:AddSheet
%
% Display name of worksheet.
%
  fprintf(['\nResults worksheet: %', num2str(length(resultsworksheet)), 's \n'], resultsworksheet); 
%  
% Assign results to write to worksheet.
%
  t1      = MeasIn.t;
  y1      = MeasIn.y;
  yObs    = Obs.y;
  t2      = Sens.t;
  y12     = interp1(t1, y1, t2);
  yObs2   = interp1(t1, yObs, t2);
  Y2      = Sens.Y;
  uY2     = Sens.uY;
  nS      = size(Y2, 2);
  yE      = MeasOut.y; 
  uyE     = MeasOut.uy;
  m2      = size(yE, 1);
  indices = MeasOut.indices; 
%
% Write headers.
%
  xlswrite(workbookpath, {'Seed:'}, resultsworksheet, 'A2');
  txt = {'Time (s)', 'yIn', 'yEst', 'u(yEst)', 'LCS', 'yObs'};
  xlswrite(workbookpath, txt, resultsworksheet, 'A6:F6');
  for is = 1:min(nS, 10)
      str  = ['ySensor', num2str(is)];
      stru = ['u(ySensor', num2str(is),')'];
      range = [char(70 + 2*is - 1), '6:', char(70 + 2*is), '6'];
      xlswrite(workbookpath, {str, stru}, resultsworksheet, range);
  end
%
% Write results.
%
  xlswrite(workbookpath, seed, resultsworksheet, 'B2:E2');
  results1 = [t2, y12, yE, uyE];
  range = ['A7:D', num2str(6 + m2)];
  xlswrite(workbookpath, results1, resultsworksheet, range);
  for iy = 1:m2
      indiy = indices{iy};
      Ni = length(indiy);
      str = '';
      for is = 1:Ni - 1
          str = cat(2, str, [num2str(indiy(is)), ',']);
      end
      if Ni > 0
          str = cat(2, str, num2str(indiy(Ni)));
      end
      indices{iy} = str;
  end
  range = ['E7:E', num2str(6 + m2)];
  xlswrite(workbookpath, indices, resultsworksheet, range);
  range = ['F7:F', num2str(6 + m2)];
  xlswrite(workbookpath, yObs2, resultsworksheet, range);
  for is = 1:min(nS, 10)
      range = [char(70 + 2*is-1), '7:', char(70 + 2*is), num2str(6 + m2)];
      results2 = [Y2(:,is), uY2(:,is)];
      xlswrite(workbookpath, results2, resultsworksheet, range);
  end
%  
% -------------------------------------------------------------------------
% End WriteObsResults.m
% -------------------------------------------------------------------------
% Data Fusion Software (Release 1)
%
% Authors: T J Esward, P M Harris, C E Matthews, H D Minh and I M Smith
% National Physical Laboratory, UK
% 
% Date: December 2011
% 
% (c) Queen's Printer and Controller of HMSO, 2011
%--------------------------------------------------------------------------